==> nnabla_rl-0.9.0-py3-none-any.whl: tests/algorithms/test_qrdqn.py <==
# Copyright 2020,2021 Sony Corporation.
# Copyright 2021 Sony Group Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import pytest

import nnabla as nn
import nnabla_rl.algorithms as A
import nnabla_rl.environments as E
from nnabla_rl.replay_buffer import ReplayBuffer


class TestQRDQN(object):
    def setup_method(self, method):
        nn.clear_parameters()

    def test_algorithm_name(self):
        dummy_env = E.DummyDiscreteImg()
        qrdqn = A.QRDQN(dummy_env)

        assert qrdqn.__name__ == 'QRDQN'

    def test_continuous_env_unsupported(self):
        '''
        Check that error occurs when training on continuous env
        '''

        dummy_env = E.DummyContinuous()
        config = A.QRDQNConfig()
        with pytest.raises(Exception):
            A.QRDQN(dummy_env, config=config)

    def test_run_online_training(self):
        '''
        Check that no error occurs when calling online training
        '''

        dummy_env = E.DummyDiscreteImg()
        config = A.QRDQNConfig()
        config.start_timesteps = 5
        config.batch_size = 5
        config.learner_update_frequency = 1
        config.target_update_frequency = 1
        qrdqn = A.QRDQN(dummy_env, config=config)

        qrdqn.train_online(dummy_env, total_iterations=10)

    def test_run_offline_training(self):
        dummy_env = E.DummyDiscreteImg()
        batch_size = 5
        config = A.QRDQNConfig()
        config.batch_size = batch_size
        config.learner_update_frequency = 1
        config.target_update_frequency = 1

        qrdqn = A.QRDQN(dummy_env, config=config)

        buffer = ReplayBuffer()
        experiences = generate_dummy_experiences(dummy_env, batch_size)
        buffer = ReplayBuffer()
        buffer.append_all(experiences)
        qrdqn.train_offline(buffer, total_iterations=5)

    def test_compute_eval_action(self):
        dummy_env = E.DummyDiscreteImg()
        qrdqn = A.QRDQN(dummy_env)

        state = dummy_env.reset()
        state = np.float32(state)
        action = qrdqn.compute_eval_action(state)

        assert action.shape == (1, )

    def test_parameter_range(self):
        with pytest.raises(ValueError):
            A.QRDQNConfig(gamma=1.1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(gamma=-0.1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(batch_size=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(replay_buffer_size=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(learner_update_frequency=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(max_explore_steps=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(learning_rate=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(initial_epsilon=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(final_epsilon=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(test_epsilon=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(num_quantiles=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(kappa=-1)

    def test_latest_iteration_state(self):
        '''
        Check that latest iteration state has the keys and values we expected
        '''

        dummy_env = E.DummyDiscreteImg()
        qrdqn = A.QRDQN(dummy_env)

        qrdqn._quantile_dist_trainer_state = {'q_loss': 0.}

        latest_iteration_state = qrdqn.latest_iteration_state
        assert 'q_loss' in latest_iteration_state['scalar']
        assert latest_iteration_state['scalar']['q_loss'] == 0.


if __name__ == "__main__":
    import sys
    sys.path.insert(0, "./")
    from testing_utils import generate_dummy_experiences
    pytest.main()
else:
    from .testing_utils import generate_dummy_experiences

==> nnabla_rl-0.10.0-py3-none-any.whl: tests/algorithms/test_qrdqn.py <==
# Copyright 2020,2021 Sony Corporation.
# Copyright 2021 Sony Group Corporation.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import numpy as np
import pytest

import nnabla as nn
import nnabla_rl.algorithms as A
import nnabla_rl.environments as E
from nnabla_rl.replay_buffer import ReplayBuffer


class TestQRDQN(object):
    def setup_method(self, method):
        nn.clear_parameters()

    def test_algorithm_name(self):
        dummy_env = E.DummyDiscreteImg()
        qrdqn = A.QRDQN(dummy_env)

        assert qrdqn.__name__ == 'QRDQN'

    def test_continuous_action_env_unsupported(self):
        '''
        Check that error occurs when training on continuous action env
        '''

        dummy_env = E.DummyContinuous()
        config = A.QRDQNConfig()
        with pytest.raises(Exception):
            A.QRDQN(dummy_env, config=config)

    def test_run_online_training(self):
        '''
        Check that no error occurs when calling online training
        '''

        dummy_env = E.DummyDiscreteImg()
        config = A.QRDQNConfig()
        config.start_timesteps = 5
        config.batch_size = 5
        config.learner_update_frequency = 1
        config.target_update_frequency = 1
        qrdqn = A.QRDQN(dummy_env, config=config)

        qrdqn.train_online(dummy_env, total_iterations=10)

    def test_run_online_training_multistep(self):
        '''
        Check that no error occurs when calling online training
        '''

        dummy_env = E.DummyDiscreteImg()
        config = A.QRDQNConfig()
        config.num_steps = 2
        config.start_timesteps = 5
        config.batch_size = 5
        config.learner_update_frequency = 1
        config.target_update_frequency = 1
        qrdqn = A.QRDQN(dummy_env, config=config)

        qrdqn.train_online(dummy_env, total_iterations=10)

    def test_run_offline_training(self):
        dummy_env = E.DummyDiscreteImg()
        batch_size = 5
        config = A.QRDQNConfig()
        config.batch_size = batch_size
        config.learner_update_frequency = 1
        config.target_update_frequency = 1

        qrdqn = A.QRDQN(dummy_env, config=config)

        buffer = ReplayBuffer()
        experiences = generate_dummy_experiences(dummy_env, batch_size)
        buffer = ReplayBuffer()
        buffer.append_all(experiences)
        qrdqn.train_offline(buffer, total_iterations=5)

    def test_compute_eval_action(self):
        dummy_env = E.DummyDiscreteImg()
        qrdqn = A.QRDQN(dummy_env)

        state = dummy_env.reset()
        state = np.float32(state)
        action = qrdqn.compute_eval_action(state)

        assert action.shape == (1, )

    def test_parameter_range(self):
        with pytest.raises(ValueError):
            A.QRDQNConfig(gamma=1.1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(gamma=-0.1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(batch_size=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(replay_buffer_size=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(learner_update_frequency=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(max_explore_steps=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(learning_rate=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(initial_epsilon=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(final_epsilon=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(test_epsilon=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(num_quantiles=-1)
        with pytest.raises(ValueError):
            A.QRDQNConfig(kappa=-1)

    def test_latest_iteration_state(self):
        '''
        Check that latest iteration state has the keys and values we expected
        '''

        dummy_env = E.DummyDiscreteImg()
        qrdqn = A.QRDQN(dummy_env)

        qrdqn._quantile_dist_trainer_state = {'q_loss': 0.}

        latest_iteration_state = qrdqn.latest_iteration_state
        assert 'q_loss' in latest_iteration_state['scalar']
        assert latest_iteration_state['scalar']['q_loss'] == 0.


if __name__ == "__main__":
    from testing_utils import generate_dummy_experiences
    pytest.main()
else:
    from ..testing_utils import generate_dummy_experiences
